@@ -15,6 +15,7 @@
 from .._error import (
     _validate_not_none,
     _validate_type_bytes,
+    _validate_encryption_unsupported,
     _ERROR_VALUE_NEGATIVE,
 )
 from .._common_conversion import (
@@ -24,7 +25,7 @@ from .._common_conversion import (
     _get_content_md5,
 )
 from .._serialization import (
-    _get_request_body_bytes_only,
+    _get_data_bytes_only,
     _add_metadata_headers,
 )
 from .._http import HTTPRequest
@@ -32,7 +33,10 @@ from ._upload_chunking import (
     _AppendBlobChunkUploader,
     _upload_blob_chunks,
 )
-from .models import _BlobTypes
+from .models import (
+    _BlobTypes,
+    ResourceProperties
+)
 from .._constants import (
     SERVICE_HOST_BASE,
     DEFAULT_PROTOCOL,
@@ -73,7 +77,7 @@ class AppendBlobService(BaseBlobService):
 
     def __init__(self, account_name=None, account_key=None, sas_token=None, 
                  is_emulated=False, protocol=DEFAULT_PROTOCOL, endpoint_suffix=SERVICE_HOST_BASE,
-                 custom_domain=None, request_session=None, connection_string=None):
+                 custom_domain=None, request_session=None, connection_string=None, socket_timeout=None):
         '''
         :param str account_name:
             The storage account name. This is used to authenticate requests 
@@ -109,11 +113,14 @@ class AppendBlobService(BaseBlobService):
             request session. See
             http://azure.microsoft.com/en-us/documentation/articles/storage-configure-connection-string/
             for the connection string format.
+        :param int socket_timeout:
+            If specified, this will override the default socket timeout. The timeout specified is in seconds.
+            See DEFAULT_SOCKET_TIMEOUT in _constants.py for the default value.
         '''
         self.blob_type = _BlobTypes.AppendBlob
         super(AppendBlobService, self).__init__(
             account_name, account_key, sas_token, is_emulated, protocol, endpoint_suffix, 
-            custom_domain, request_session, connection_string)
+            custom_domain, request_session, connection_string, socket_timeout)
 
     def create_blob(self, container_name, blob_name, content_settings=None,
                     metadata=None, lease_id=None,
@@ -166,9 +173,11 @@ class AppendBlobService(BaseBlobService):
         '''
         _validate_not_none('container_name', container_name)
         _validate_not_none('blob_name', blob_name)
+        _validate_encryption_unsupported(self.require_encryption, self.key_encryption_key)
+
         request = HTTPRequest()
         request.method = 'PUT'
-        request.host = self._get_host()
+        request.host_locations = self._get_host_locations()
         request.path = _get_path(container_name, blob_name)
         request.query = {'timeout': _int_to_str(timeout)}
         request.headers = {
@@ -183,8 +192,7 @@ class AppendBlobService(BaseBlobService):
         if content_settings is not None:
             request.headers.update(content_settings._to_headers())
 
-        response = self._perform_request(request)
-        return _parse_base_properties(response)
+        return self._perform_request(request, _parse_base_properties)
 
     def append_block(self, container_name, blob_name, block,
                      validate_content=False, maxsize_condition=None,
@@ -213,14 +221,14 @@ class AppendBlobService(BaseBlobService):
             the append blob. If the Append Block operation would cause the blob
             to exceed that limit or if the blob size is already greater than the
             value specified in this header, the request will fail with
-            MaxBlobSizeConditionNotMet error (HTTP status code 412 – Precondition Failed).
+            MaxBlobSizeConditionNotMet error (HTTP status code 412 - Precondition Failed).
         :param int appendpos_condition:
             Optional conditional header, used only for the Append Block operation.
             A number indicating the byte offset to compare. Append Block will
             succeed only if the append position is equal to this number. If it
             is not, the request will fail with the
             AppendPositionConditionNotMet error
-            (HTTP status code 412 – Precondition Failed).
+            (HTTP status code 412 - Precondition Failed).
         :param str lease_id:
             Required if the blob has an active lease.
         :param datetime if_modified_since:
@@ -254,9 +262,11 @@ class AppendBlobService(BaseBlobService):
         _validate_not_none('container_name', container_name)
         _validate_not_none('blob_name', blob_name)
         _validate_not_none('block', block)
+        _validate_encryption_unsupported(self.require_encryption, self.key_encryption_key)
+
         request = HTTPRequest()
         request.method = 'PUT'
-        request.host = self._get_host()
+        request.host_locations = self._get_host_locations()
         request.path = _get_path(container_name, blob_name)
         request.query = {
             'comp': 'appendblock',
@@ -271,21 +281,19 @@ class AppendBlobService(BaseBlobService):
             'If-Match': _to_str(if_match),
             'If-None-Match': _to_str(if_none_match)
         }
-        request.body = _get_request_body_bytes_only('block', block)
+        request.body = _get_data_bytes_only('block', block)
 
         if validate_content:
             computed_md5 = _get_content_md5(request.body)
             request.headers['Content-MD5'] = _to_str(computed_md5)
 
-        response = self._perform_request(request)
-        return _parse_append_block(response)
+        return self._perform_request(request, _parse_append_block)
 
     #----Convenience APIs----------------------------------------------
 
     def append_blob_from_path(
         self, container_name, blob_name, file_path, validate_content=False,
-        maxsize_condition=None, progress_callback=None,
-        max_retries=5, retry_wait=1.0, lease_id=None, timeout=None):
+        maxsize_condition=None, progress_callback=None, lease_id=None, timeout=None):
         '''
         Appends to the content of an existing blob from a file path, with automatic
         chunking and progress notifications.
@@ -308,46 +316,43 @@ class AppendBlobService(BaseBlobService):
             the append blob. If the Append Block operation would cause the blob
             to exceed that limit or if the blob size is already greater than the
             value specified in this header, the request will fail with
-            MaxBlobSizeConditionNotMet error (HTTP status code 412 – Precondition Failed).
+            MaxBlobSizeConditionNotMet error (HTTP status code 412 - Precondition Failed).
         :param progress_callback:
             Callback for progress with signature function(current, total) where
             current is the number of bytes transfered so far, and total is the
             size of the blob, or None if the total size is unknown.
         :type progress_callback: callback function in format of func(current, total)
-        :param int max_retries:
-            Number of times to retry upload of blob chunk if an error occurs.
-        :param int retry_wait:
-            Sleep time in secs between retries.
         :param str lease_id:
             Required if the blob has an active lease.
         :param int timeout:
             The timeout parameter is expressed in seconds. This method may make 
             multiple calls to the Azure service and the timeout will apply to 
             each call individually.
+        :return: ETag and last modified properties for the Append Blob
+        :rtype: :class:`~azure.storage.blob.models.ResourceProperties`
         '''
         _validate_not_none('container_name', container_name)
         _validate_not_none('blob_name', blob_name)
         _validate_not_none('file_path', file_path)
+        _validate_encryption_unsupported(self.require_encryption, self.key_encryption_key)
 
         count = path.getsize(file_path)
         with open(file_path, 'rb') as stream:
-            self.append_blob_from_stream(
-                container_name,
-                blob_name,
-                stream,
-                count=count,
-                validate_content=validate_content,
-                maxsize_condition=maxsize_condition,
-                progress_callback=progress_callback,
-                max_retries=max_retries,
-                retry_wait=retry_wait,
-                lease_id=lease_id,
-                timeout=timeout)
+            return self.append_blob_from_stream(
+                    container_name,
+                    blob_name,
+                    stream,
+                    count=count,
+                    validate_content=validate_content,
+                    maxsize_condition=maxsize_condition,
+                    progress_callback=progress_callback,
+                    lease_id=lease_id,
+                    timeout=timeout)
 
     def append_blob_from_bytes(
         self, container_name, blob_name, blob, index=0, count=None,
         validate_content=False, maxsize_condition=None, progress_callback=None,
-        max_retries=5, retry_wait=1.0, lease_id=None, timeout=None):
+        lease_id=None, timeout=None):
         '''
         Appends to the content of an existing blob from an array of bytes, with
         automatic chunking and progress notifications.
@@ -375,28 +380,27 @@ class AppendBlobService(BaseBlobService):
             the append blob. If the Append Block operation would cause the blob
             to exceed that limit or if the blob size is already greater than the
             value specified in this header, the request will fail with
-            MaxBlobSizeConditionNotMet error (HTTP status code 412 – Precondition Failed).
+            MaxBlobSizeConditionNotMet error (HTTP status code 412 - Precondition Failed).
         :param progress_callback:
             Callback for progress with signature function(current, total) where
             current is the number of bytes transfered so far, and total is the
             size of the blob, or None if the total size is unknown.
         :type progress_callback: callback function in format of func(current, total)
-        :param int max_retries:
-            Number of times to retry upload of blob chunk if an error occurs.
-        :param int retry_wait:
-            Sleep time in secs between retries.
         :param str lease_id:
             Required if the blob has an active lease.
         :param int timeout:
             The timeout parameter is expressed in seconds. This method may make 
             multiple calls to the Azure service and the timeout will apply to 
             each call individually.
+        :return: ETag and last modified properties for the Append Blob
+        :rtype: :class:`~azure.storage.blob.models.ResourceProperties`
         '''
         _validate_not_none('container_name', container_name)
         _validate_not_none('blob_name', blob_name)
         _validate_not_none('blob', blob)
         _validate_not_none('index', index)
         _validate_type_bytes('blob', blob)
+        _validate_encryption_unsupported(self.require_encryption, self.key_encryption_key)
 
         if index < 0:
             raise IndexError(_ERROR_VALUE_NEGATIVE.format('index'))
@@ -407,23 +411,21 @@ class AppendBlobService(BaseBlobService):
         stream = BytesIO(blob)
         stream.seek(index)
 
-        self.append_blob_from_stream(
-            container_name,
-            blob_name,
-            stream,
-            count=count,
-            validate_content=validate_content,
-            maxsize_condition=maxsize_condition,
-            lease_id=lease_id,
-            progress_callback=progress_callback,
-            max_retries=max_retries,
-            retry_wait=retry_wait,
-            timeout=timeout)
+        return self.append_blob_from_stream(
+                container_name,
+                blob_name,
+                stream,
+                count=count,
+                validate_content=validate_content,
+                maxsize_condition=maxsize_condition,
+                lease_id=lease_id,
+                progress_callback=progress_callback,
+                timeout=timeout)
 
     def append_blob_from_text(
         self, container_name, blob_name, text, encoding='utf-8',
         validate_content=False, maxsize_condition=None, progress_callback=None,
-        max_retries=5, retry_wait=1.0, lease_id=None, timeout=None):
+        lease_id=None, timeout=None):
         '''
         Appends to the content of an existing blob from str/unicode, with
         automatic chunking and progress notifications.
@@ -448,49 +450,46 @@ class AppendBlobService(BaseBlobService):
             the append blob. If the Append Block operation would cause the blob
             to exceed that limit or if the blob size is already greater than the
             value specified in this header, the request will fail with
-            MaxBlobSizeConditionNotMet error (HTTP status code 412 – Precondition Failed).
+            MaxBlobSizeConditionNotMet error (HTTP status code 412 - Precondition Failed).
         :param progress_callback:
             Callback for progress with signature function(current, total) where
             current is the number of bytes transfered so far, and total is the
             size of the blob, or None if the total size is unknown.
         :type progress_callback: callback function in format of func(current, total)
-        :param int max_retries:
-            Number of times to retry upload of blob chunk if an error occurs.
-        :param int retry_wait:
-            Sleep time in secs between retries.
         :param str lease_id:
             Required if the blob has an active lease.
         :param int timeout:
             The timeout parameter is expressed in seconds. This method may make 
             multiple calls to the Azure service and the timeout will apply to 
             each call individually.
+        :return: ETag and last modified properties for the Append Blob
+        :rtype: :class:`~azure.storage.blob.models.ResourceProperties`
         '''
         _validate_not_none('container_name', container_name)
         _validate_not_none('blob_name', blob_name)
         _validate_not_none('text', text)
+        _validate_encryption_unsupported(self.require_encryption, self.key_encryption_key)
 
         if not isinstance(text, bytes):
             _validate_not_none('encoding', encoding)
             text = text.encode(encoding)
 
-        self.append_blob_from_bytes(
-            container_name,
-            blob_name,
-            text,
-            index=0,
-            count=len(text),
-            validate_content=validate_content,
-            maxsize_condition=maxsize_condition,
-            lease_id=lease_id,
-            progress_callback=progress_callback,
-            max_retries=max_retries,
-            retry_wait=retry_wait,
-            timeout=timeout)
+        return self.append_blob_from_bytes(
+                container_name,
+                blob_name,
+                text,
+                index=0,
+                count=len(text),
+                validate_content=validate_content,
+                maxsize_condition=maxsize_condition,
+                lease_id=lease_id,
+                progress_callback=progress_callback,
+                timeout=timeout)
 
     def append_blob_from_stream(
         self, container_name, blob_name, stream, count=None,
         validate_content=False, maxsize_condition=None, progress_callback=None, 
-        max_retries=5, retry_wait=1.0, lease_id=None, timeout=None):
+        lease_id=None, timeout=None):
         '''
         Appends to the content of an existing blob from a file/stream, with
         automatic chunking and progress notifications.
@@ -516,27 +515,30 @@ class AppendBlobService(BaseBlobService):
             the append blob. If the Append Block operation would cause the blob
             to exceed that limit or if the blob size is already greater than the
             value specified in this header, the request will fail with
-            MaxBlobSizeConditionNotMet error (HTTP status code 412 – Precondition Failed).
+            MaxBlobSizeConditionNotMet error (HTTP status code 412 - Precondition Failed).
         :param progress_callback:
             Callback for progress with signature function(current, total) where
             current is the number of bytes transfered so far, and total is the
             size of the blob, or None if the total size is unknown.
         :type progress_callback: callback function in format of func(current, total)
-        :param int max_retries:
-            Number of times to retry upload of blob chunk if an error occurs.
-        :param int retry_wait:
-            Sleep time in secs between retries.
         :param str lease_id:
             Required if the blob has an active lease.
         :param int timeout:
             The timeout parameter is expressed in seconds. This method may make 
             multiple calls to the Azure service and the timeout will apply to 
             each call individually.
+        :return: ETag and last modified properties for the Append Blob
+        :rtype: :class:`~azure.storage.blob.models.ResourceProperties`
         '''
         _validate_not_none('container_name', container_name)
         _validate_not_none('blob_name', blob_name)
         _validate_not_none('stream', stream)
+        _validate_encryption_unsupported(self.require_encryption, self.key_encryption_key)
 
+        # _upload_blob_chunks returns the block ids for block blobs so resource_properties
+        # is passed as a parameter to get the last_modified and etag for page and append blobs.
+        # this info is not needed for block_blobs since _put_block_list is called after which gets this info
+        resource_properties = ResourceProperties()
         _upload_blob_chunks(
             blob_service=self,
             container_name=container_name,
@@ -545,12 +547,13 @@ class AppendBlobService(BaseBlobService):
             block_size=self.MAX_BLOCK_SIZE,
             stream=stream,
             max_connections=1, # upload not easily parallelizable
-            max_retries=max_retries,
-            retry_wait=retry_wait,
             progress_callback=progress_callback,
             validate_content=validate_content,
             lease_id=lease_id,
             uploader_class=_AppendBlobChunkUploader,
             maxsize_condition=maxsize_condition,
-            timeout=timeout
-        )
\ No newline at end of file
+            timeout=timeout,
+            resource_properties=resource_properties
+        )
+
+        return resource_properties
\ No newline at end of file
